#include "../res.h"
#include "mem.h"
#include "mode3.h"

//---------------------------------------------------------------------------
ST_MODE3 Mode3;


// ../libgba/common.c
extern u32 dummy;

//---------------------------------------------------------------------------
EWRAM_CODE void Mode3Init()
{
	_Memset(&Mode3,  0x00, sizeof(ST_MODE3));

	Mode3MallocBuf();
/*
	Mode3.pDat  = (u16*)&mplus_s10dBitmap;
	Mode3.imgCx = MPLUS_S10D_IMG_CX;
	Mode3.imgCy = MPLUS_S10D_IMG_CY;
	Mode3.scnCx = SCREEN_WIDTH  / MPLUS_S10D_IMG_CX;
	Mode3.scnCy = SCREEN_HEIGHT / MPLUS_S10D_IMG_CY;
*/

	REG_DISPCNT = (MODE_3 | BG2_ON);
}
//---------------------------------------------------------------------------
EWRAM_CODE void Mode3MallocBuf()
{
	if(Mode3.pBuf != NULL)
	{
		Mode3FreeBuf();
	}

	Mode3.pBuf = MemMalloc(MODE3_MAX_BUF_SIZE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void Mode3FreeBuf()
{
	MemFree((u32)Mode3.pBuf);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3Update()
{
	if(Mode3.isDraw == TRUE)
	{
		CpuSet(Mode3.pBuf, (void*)VRAM, (MODE3_MAX_BUF_SIZE / 4) | CPUSET_32BIT);
		Mode3.isDraw = FALSE;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3Cls()
{
	CpuSet(&dummy, Mode3.pBuf, (MODE3_MAX_BUF_SIZE / 4) | CPUSET_32BIT | CPUSET_SRC_FIX);

	Mode3.curX   = 0;
	Mode3.curY   = 0;
	Mode3.isDraw = TRUE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawFull(u16* pBuf)
{
	CpuSet(pBuf, Mode3.pBuf, (MODE3_MAX_BUF_SIZE / 4) | CPUSET_32BIT);

	Mode3.curX   = 0;
	Mode3.curY   = 0;
	Mode3.isDraw = TRUE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawFullLz77(u16* pBuf)
{
	LZ77UnCompWram(pBuf, Mode3.pBuf);

	Mode3.curX   = 0;
	Mode3.curY   = 0;
	Mode3.isDraw = TRUE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawPset(u16 x, u16 y, u16 col)
{
	if(x  >= SCREEN_WIDTH)  return;
	if(y  >= SCREEN_HEIGHT) return;

	u16* pScn = Mode3.pBuf + x + (y * SCREEN_WIDTH);
	*pScn = col;

	Mode3.isDraw = TRUE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawLine(u16 x1, u16 y1, u16 x2, u16 y2, u16 col)
{
	s16 dx, dy;
	s16 sx, sy;
	s16 x, y;
	s16 E;
	s16 i;

	sx = ( x2 > x1 ) ? 1 : -1;
	dx = ( x2 > x1 ) ? x2 - x1 : x1 - x2;
	sy = ( y2 > y1 ) ? 1 : -1;
	dy = ( y2 > y1 ) ? y2 - y1 : y1 - y2;

	x = x1;
	y = y1;

	if(dx >= dy)
	{
		//X1ȉ̏ꍇ
		E = -dx;

		for(i=0; i<=dx; i++)
		{
			Mode3DrawPset(x, y, col);

			x += sx;
			E += 2 * dy;

			if(E >= 0)
			{
				y += sy;
				E -= 2 * dx;
			}
		}
	}
	else
	{
		//X1傫ꍇ
		E = -dy;
		
		for(i=0; i<=dy; i++)
		{
			Mode3DrawPset(x, y, col);

			y += sy;
			E += 2 * dx;

			if(E >= 0)
			{
				x += sx;
				E -= 2 * dy;
			}
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawCircle(u16 sx, u16 sy, u16 rad, u16 col)
{
	u16 x = rad;
	u16 y = 0;
	s16 p = 3 - 2 * rad;

	while(x >= y)
	{
		Mode3DrawPset(sx + x, sy + y, col);
		Mode3DrawPset(sx - x, sy + y, col);
		Mode3DrawPset(sx + x, sy - y, col);
		Mode3DrawPset(sx - x, sy - y, col);
		Mode3DrawPset(sx + y, sy + x, col);
		Mode3DrawPset(sx - y, sy + x, col);
		Mode3DrawPset(sx + y, sy - x, col);
		Mode3DrawPset(sx - y, sy - x, col);

		if(p >= 0)
		{
			x--;
			p -= 4 * x;
		}

		y++;
		p += 4 * y + 2;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawStr(char* str)
{
	s16 len = _Strlen(str);
	s16 i;

	for(i=0; i<len; i++)
	{
		Mode3DrawChr(str[i]);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawStrLf(char* str)
{
	Mode3DrawStr(str);

	Mode3.curX = 0;
	Mode3.curY++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawChr(char chr)
{
	ST_MODE3* p = &Mode3;

	if(p->curX >= p->scnCx)
	{
		p->curX = 0;
		p->curY++;
	}

	if(p->curY >= p->scnCy)
	{
		p->curX = 0;
		p->curY = 0;

		Mode3Cls();
	}

	// TODO _ASSERT(chr >= 0x20 && char <= 0x7e && "chr error")

	u16* pFont = p->pDat + (p->imgCx * p->imgCy) * (chr - 0x20);
	s16  x, y;

	for(y=0; y<p->imgCy; y++)
	{
		for(x=0; x<p->imgCx; x++)
		{
			Mode3DrawPset((p->curX * p->imgCx) + x, (p->curY * p->imgCy) + y, *pFont++);
		}
	}

	p->curX++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawNum(s16 num)
{
	char buf[80];

	_Sprintf(buf, "%d", num);
	Mode3DrawStr(buf);
}
//---------------------------------------------------------------------------
EWRAM_CODE void Mode3SetLocate(u16 x, u16 y)
{
	Mode3.curX = x;
	Mode3.curY = y;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 Mode3GetScnCy()
{
	return Mode3.scnCy;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 Mode3GetImgCy()
{
	return Mode3.imgCy;
}
