#include "state.h"
#include "libmy/my.h"
#include "irq_arm.h"
#include "game.h"


//---------------------------------------------------------------------------
u16 State;


//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	REG_WSCNT = 0x4317;
	_Srand(0x1234567);

	State = STATE_INIT;
}
//---------------------------------------------------------------------------
IWRAM_CODE void StateExec()
{
	for(;;)
	{
		switch(State)
		{
		case STATE_INIT:    StateExecInit();    break;
		case STATE_RESET:   StateExecReset();   break;
		case STATE_RUN:     StateExecRun();     break;
		case STATE_RESTART: StateExecRestart(); break;

		default:
			SystemError("[Err] StateMain State");
			break;
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void StateExecInit()
{
	MemInit();
//	GbfsInit();

//	BgInit();
	Mode3Init();

	SprInit();
	KeyInit();
	DmaInit();
	LexInit();
	SndInit();
	FadeInit();
	AdrvInit();
	DebugInit();

	IntrInit();
	IrqInit();
	IntrStart();

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
IWRAM_CODE void StateExecReset()
{
	GameInit();

	State = STATE_RUN;
}
//---------------------------------------------------------------------------
IWRAM_CODE void StateExecRun()
{
	for(;;)
	{
		SystemCall(5);

		// Dma
//		DmaTransfer();

		// Update
		Mode3Update();
//		SprUpdate();

		// Draw
		GameDraw();
//		DebugDraw();

		// Exec
		KeyExec();
		GameExec();
//		DebugExec();
	}

	State = STATE_RESTART;
}
//---------------------------------------------------------------------------
IWRAM_CODE void StateExecRestart()
{
//	FadeIn();
//	FadeOut();

	State = STATE_RUN;
}
